﻿using Microsoft.IdentityModel.Clients.ActiveDirectory;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace Microsoft.Samples.AuthenticationManagement
{
    /// <summary>
    /// Implementation interface IAuthManager using Azure Active Library for .Net
    /// You can implement your own AuthManager 
    /// 
    /// </summary>
    public class AuthManager : IAuthManager
    {
        public readonly AuthTokenStorage TokenStorage;

        /// <summary>
        /// contructor
        /// </summary>
        /// <param name="tokenStorage"></param>
        public AuthManager(AuthTokenStorage tokenStorage)
        {
            this.TokenStorage = tokenStorage;
        }

        /// <summary>
        /// Method for obtaining Azure AD Access token by prompting user to login, the application will impersonlize under the user security context
        /// </summary>
        /// <param name="clientId">client Id of the application registered in Azure AD</param>
        /// <param name="resource"></param>
        /// <returns></returns>
        public async Task AcquireTokens(string clientId, string tenantId, string resource)
        {
            var tokenCache = this.TokenStorage.GetCache();
            if (string.IsNullOrEmpty(tenantId))
                tenantId = Constants.AzureADTenantId;
            var cacheInfo = await GetAuthorizationResult(tokenCache, clientId, tenantId, resource, LoginType.Interactive);

            this.TokenStorage.SaveCache(tokenCache);
        }

        /// <summary>
        /// Clear the token storage
        /// </summary>
        public void ClearTokenCache()
        {
            this.TokenStorage.ClearCache();
        }

        /// <summary>
        /// Dump token cache
        /// </summary>
        /// <param name="resource"></param>
        /// <returns></returns>
        public IEnumerable<string> DumpTokenCache(string resource)
        {
            var tokenCache = this.TokenStorage.GetCache();
            foreach(var cacheItem in tokenCache.GetValues(resource))
            {
                var tenantId = cacheItem.TenantId;

                if(!string.IsNullOrEmpty(cacheItem.DisplayableId))
                {
                    yield return string.Format("User: {0}", cacheItem.DisplayableId);
                }
                else if(!string.IsNullOrEmpty(cacheItem.ApplicationId))
                {
                    yield return string.Format("Application: {0}, Tenant: {1}", cacheItem.ApplicationId, tenantId);
                }
                else
                {
                    throw new NotImplementedException();
                }
                yield return string.Empty;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="clientId"></param>
        /// <param name="tenantId"></param>
        /// <param name="resource"></param>
        /// <returns></returns>
        public async Task<AuthTokenCacheInfo> GetToken(string clientId, string tenantId, string resource)
        {
            var tokenCache = this.TokenStorage.GetCache();
            AuthTokenCacheInfo cacheInfo = null;
            if (!tokenCache.TryGetValue(tenantId, resource,LoginType.Interactive, out cacheInfo))
            {
                return await GetRecentToken(clientId, resource);
            }

            if(cacheInfo.ExpiresOn<=DateTimeOffset.UtcNow)
            {
                cacheInfo = await RefreshToken(tokenCache, clientId, cacheInfo);
                this.TokenStorage.SaveCache(tokenCache);
            }

            this.TokenStorage.SaveRecentToken(cacheInfo);

            return cacheInfo;
        }

        /// <summary>
        /// Method to aquire access token by service principal or client credential, basically using application without any user interaction
        /// </summary>
        /// <param name="tenantId"></param>
        /// <param name="appId"></param>
        /// <param name="appKey"></param>
        /// <param name="resource"></param>
        /// <returns></returns>
        public AuthTokenCacheInfo GetTokenBySpn(string tenantId, string appId, string appKey, string resource)
        {
            var tokenCache = this.TokenStorage.GetCache();
            var cacheinfo = GetAuthorizationResultBySpn(tokenCache, tenantId, appId, appKey, resource);
            this.TokenStorage.SaveCache(tokenCache);

            return cacheinfo;
        }

        /// <summary>
        /// Method for obtaining access token using username, and password. The Azure AD Library .Net implements this option with limitations
        /// You can read more here: http://www.cloudidentity.com/blog/2014/07/08/using-adal-net-to-authenticate-users-via-usernamepassword/
        /// </summary>
        /// <param name="clientId"></param>
        /// <param name="username"></param>
        /// <param name="password"></param>
        /// <param name="resource"></param>
        /// <returns></returns>
        public AuthTokenCacheInfo GetTokenByUpn(string clientId, string username, string password, string resource)
        {
            var tokenCache = this.TokenStorage.GetCache();
            var cacheinfo = GetAuthorizationResultByUpn(tokenCache, clientId, Constants.AzureADTenantId, username, password, resource);
            this.TokenStorage.SaveCache(tokenCache);

            return cacheinfo;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="clientId"></param>
        /// <param name="resource"></param>
        /// <returns></returns>
        protected async Task<AuthTokenCacheInfo> GetRecentToken(string clientId, string resource)
        {
            AuthTokenCacheInfo cacheInfo = this.TokenStorage.GetRecentToken();

            if (cacheInfo != null && cacheInfo.ExpiresOn<=DateTimeOffset.UtcNow)
            {
                var tokenCache = this.TokenStorage.GetCache();
                cacheInfo = await RefreshToken(tokenCache, clientId, cacheInfo);
                this.TokenStorage.SaveCache(tokenCache);
            }

            return cacheInfo;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="tokenCache"></param>
        /// <param name="clientId"></param>
        /// <param name="cacheInfo"></param>
        /// <returns></returns>
        protected async Task<AuthTokenCacheInfo> RefreshToken(AuthTokenCache tokenCache, string clientId, AuthTokenCacheInfo cacheInfo)
        {
            if(!string.IsNullOrEmpty(cacheInfo.RefreshToken))
            {
                try
                {
                    return await GetAuthorizationResultByRefreshToken(tokenCache, clientId, cacheInfo);
                }
                catch(AdalServiceException ex)
                {
                    if(ex.Message.IndexOf("The provided access grant is expired or revoked")>0)
                    {
                        AcquireTokens(clientId, cacheInfo.TenantId, cacheInfo.Resource).Wait();
                        cacheInfo = GetToken(clientId, cacheInfo.TenantId, cacheInfo.Resource).Result;
                        tokenCache.Clone(this.TokenStorage.GetCache());
                        return cacheInfo;
                    }

                    throw;
                }
            }
            else if(!string.IsNullOrEmpty(cacheInfo.ApplicationId)&& !string.IsNullOrEmpty(cacheInfo.ApplicationKey))
            {
                return GetAuthorizationResultBySpn(tokenCache, cacheInfo.TenantId, cacheInfo.ApplicationId, cacheInfo.ApplicationKey, cacheInfo.Resource);
            }

            throw new NotImplementedException();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public bool IsCacheValid()
        {
            return this.TokenStorage.IsCacheValid();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="tokenCache"></param>
        /// <param name="clientId"></param>
        /// <param name="cacheInfo"></param>
        /// <returns></returns>
        protected async Task<AuthTokenCacheInfo> GetAuthorizationResultByRefreshToken(AuthTokenCache tokenCache, string clientId, AuthTokenCacheInfo cacheInfo)
        {
            var authority = string.Format("{0}/{1}", Constants.AzureADLoginUrl, cacheInfo.TenantId);
            var context = new AuthenticationContext(
                authority: authority,
                validateAuthority: true,
                tokenCache: tokenCache);
            AuthenticationResult result = await context.AcquireTokenByRefreshTokenAsync(
                refreshToken: cacheInfo.RefreshToken,
                clientId: clientId,
                resource: cacheInfo.Resource);

            var ret = new AuthTokenCacheInfo(cacheInfo.Resource, cacheInfo.LoginType, result);
            ret.TenantId = cacheInfo.TenantId;
            ret.DisplayableId = cacheInfo.DisplayableId;
            tokenCache.Add(ret);
            return ret;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="tokenCache"></param>
        /// <param name="clientId"></param>
        /// <param name="tenantId"></param>
        /// <param name="resource"></param>
        /// <returns></returns>
        protected Task<AuthTokenCacheInfo> GetAuthorizationResult(AuthTokenCache tokenCache, string clientId, string tenantId, string resource, LoginType loginType)
        {
            var tcs = new TaskCompletionSource<AuthTokenCacheInfo>();

            AuthTokenCacheInfo found = null;
            if(tokenCache.TryGetValue(tenantId, resource, loginType, out found))
            {
                tcs.SetResult(found);
                return tcs.Task;
            }

            var thread = new Thread(() =>
            {
                try
                {
                    var authority = String.Format("{0}/{1}", Constants.AzureADLoginUrl, tenantId);
                    var context = new AuthenticationContext(
                        authority: authority,
                        validateAuthority: true,
                        tokenCache: tokenCache);

                    AuthenticationResult result = null;
                    
                    result = context.AcquireToken(
                        resource: resource,
                        clientId: clientId,
                        redirectUri: new Uri(Constants.AzureADRedirectUri),
                        promptBehavior: PromptBehavior.Always);

                    var cacheInfo = new AuthTokenCacheInfo(resource, loginType, result);
                    tokenCache.Add(cacheInfo);
                    tcs.TrySetResult(cacheInfo);
                }
                catch (Exception ex)
                {
                    tcs.TrySetException(ex);
                }
            });

            thread.SetApartmentState(ApartmentState.STA);
            thread.Name = "AcquireTokenThread";
            thread.Start();

            return tcs.Task;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="tokenCache"></param>
        /// <param name="tenantId"></param>
        /// <param name="appId"></param>
        /// <param name="appKey"></param>
        /// <param name="resource"></param>
        /// <returns></returns>
        protected AuthTokenCacheInfo GetAuthorizationResultBySpn(AuthTokenCache tokenCache, string tenantId, string appId, string appKey, string resource)
        {
            AuthTokenCacheInfo found;
            if (tokenCache.TryGetValue(tenantId, resource, LoginType.Spn, out found))
            {
                return found;
            }

            
            var authority = String.Format("{0}/{1}", Constants.AzureADLoginUrl, tenantId);
            var context = new AuthenticationContext(
                authority: authority,
                validateAuthority: true,
                tokenCache: tokenCache);
            var credential = new ClientCredential(appId, appKey);
            var result = context.AcquireToken(resource, credential);

            var cacheInfo = new AuthTokenCacheInfo(tenantId, appId, appKey, resource, LoginType.Spn, result);
            tokenCache.Add(cacheInfo);
            return cacheInfo;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="tokenCache"></param>
        /// <param name="clientId"></param>
        /// <param name="tenantId"></param>
        /// <param name="username"></param>
        /// <param name="password"></param>
        /// <param name="resource"></param>
        /// <returns></returns>
        protected AuthTokenCacheInfo GetAuthorizationResultByUpn(AuthTokenCache tokenCache, string clientId, string tenantId, string username, string password, string resource)
        {
            AuthTokenCacheInfo found;
            if (tokenCache.TryGetValue(tenantId, resource, LoginType.Upn, out found))
            {
                return found;
            }

            
            var authority = String.Format("{0}/{1}", Constants.AzureADLoginUrl, tenantId);
            var context = new AuthenticationContext(
                authority: authority,
                validateAuthority: true,
                tokenCache: tokenCache);
            var credential = new UserCredential(username, password);
            var result = context.AcquireToken(resource, clientId, credential);

            var cacheInfo = new AuthTokenCacheInfo(resource, LoginType.Upn, result);
            tokenCache.Add(cacheInfo);
            return cacheInfo;
        }
    }
}
