﻿using Microsoft.Samples.CloudSolutionProviderManagement.PartnerCenter.DataModel;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;

namespace Microsoft.Samples.CloudSolutionProviderManagement.PartnerCenter
{
    /// <summary>
    /// Static Class Encapulates all partnercenter related APIs invocations
    /// </summary>
    /// All methods are implemented using HttpClient class to demonstrate how to invoke Partner Center RESTful APIs directly so 
    /// the implementation could be easily referenced in different programming languages
    /// API Document: https://msdn.microsoft.com/en-us/library/partnercenter/dn974944.aspx
    public static class PartnerCenterManager
    {
        #region Partner Center APIs Auth Token Handling Methods
        public static async Task<PartnerCenterAuthToken> GetSalesAgentToken(PartnerCenterAuthToken accessToken, DelegatingHandler handler)
        {
            PartnerCenterAuthToken authToken = null;
            try
            {
                var path = string.Format("{0}/my-org/tokens", Constants.CloudSolutionProviderAPIsUrl);
                Uri uri = new Uri(path);
                HttpContent payload = new StringContent("grant_type=client_credentials", Encoding.UTF8, "application/x-www-form-urlencoded");
                using (HttpResponseMessage response = await PartnerCenterApiInvoke(uri, accessToken, "Post", handler, payload, null))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        AuthToken token= JsonConvert.DeserializeObject<AuthToken>(jsonResult);
                        authToken = new PartnerCenterAuthToken(token.AccessToken, DateTimeOffset.UtcNow.AddSeconds((double)token.ExpiresIn));
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch(Exception ex)
            {
                throw ex;
            }

            return authToken;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="accessToken"></param>
        /// <param name="customerId">each customer in partner center is assigned a customer id, it is different than tenant id</param>
        /// <param name="handler"></param>
        /// <returns></returns>
        public static async Task<PartnerCenterAuthToken> GetCustomerAgentToken(PartnerCenterAuthToken accessToken, string customerId, DelegatingHandler handler)
        {
            PartnerCenterAuthToken authToken = null;
            try
            {
                var path = string.Format("{0}/{1}/tokens", Constants.CloudSolutionProviderAPIsUrl, customerId);
                Uri uri = new Uri(path);
                HttpContent payload = new StringContent("grant_type=client_credentials", Encoding.UTF8, "application/x-www-form-urlencoded");
                using (HttpResponseMessage response = await PartnerCenterApiInvoke(uri, accessToken, "Post", handler, payload, null))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        AuthToken token = JsonConvert.DeserializeObject<AuthToken>(jsonResult);
                        authToken = new PartnerCenterAuthToken(token.AccessToken, DateTimeOffset.UtcNow.AddSeconds((double)token.ExpiresIn));
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }

            return authToken;
        }
        #endregion

        #region Customers Management Related Methods
        public static async Task<Reseller> GetReseller(PartnerCenterAuthToken saToken, DelegatingHandler handler, string tenantId)
        {
            Reseller reseller = null;
            try
            {
                var path = string.Format("{0}/customers/get-by-identity?provider=AAD&type=tenant&tid={1}", Constants.CloudSolutionProviderAPIsUrl, tenantId);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await PartnerCenterApiInvoke(uri, saToken, "Get", handler, payload, null))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        reseller = JsonConvert.DeserializeObject<Reseller>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch(Exception ex)
            {
                throw ex;
            }
            return reseller;
        }

        /// <summary>
        /// method for obtaining customer information
        /// </summary>
        /// <param name="saToken"></param>
        /// <param name="handler"></param>
        /// <param name="customerTenantId"></param>
        /// <param name="resellerTenantId"></param>
        /// <returns></returns>
        public static async Task<Customer> GetCustomer(PartnerCenterAuthToken saToken, DelegatingHandler handler, string customerTenantId, string resellerTenantId)
        {
            Customer customer = null;
            try
            {
                var path = string.Format("{0}/customers/get-by-identity?provider=AAD&type=external_group&tid={1}&etid={2}", Constants.CloudSolutionProviderAPIsUrl, customerTenantId, resellerTenantId);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await PartnerCenterApiInvoke(uri, saToken, "Get", handler, payload, null))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        customer = JsonConvert.DeserializeObject<Customer>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return customer;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="saToken"></param>
        /// <param name="handler"></param>
        /// <param name="customerCid"></param>
        /// <returns></returns>
        public static async Task<Customer> GetCustomerById(PartnerCenterAuthToken saToken, DelegatingHandler handler, string customerCid)
        {
            Customer customer = null;
            try
            {
                var path = string.Format("{0}/customers/{1}", Constants.CloudSolutionProviderAPIsUrl, customerCid);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await PartnerCenterApiInvoke(uri, saToken, "Get", handler, payload, null))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        customer = JsonConvert.DeserializeObject<Customer>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return customer;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="saToken"></param>
        /// <param name="handler"></param>
        /// <param name="resellerCid"></param>
        /// <param name="newCustomer"></param>
        /// <returns></returns>
        public static async Task<Customer> NewCustomer(PartnerCenterAuthToken saToken, DelegatingHandler handler, string resellerCid, NewCustomer newCustomer)
        {
            Customer customer = null;
            try
            {
                var path = string.Format("{0}/{1}/customers/create-reseller-customer", Constants.CloudSolutionProviderAPIsUrl, resellerCid);
                Uri uri = new Uri(path);
                HttpContent payload = new StringContent(JsonConvert.SerializeObject(newCustomer), Encoding.UTF8, Constants.JsonContentType);
                using (HttpResponseMessage response = await PartnerCenterApiInvoke(uri, saToken, "Post", handler, payload, null))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        customer = JsonConvert.DeserializeObject<Customer>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return customer;
        }
        #endregion

        #region Orders Management Related Methods
        /// <summary>
        /// 
        /// </summary>
        /// <param name="saToken"></param>
        /// <param name="handler"></param>
        /// <param name="resellerCid"></param>
        /// <param name="customerCid"></param>
        /// <returns></returns>
        public static async Task<Orders> GetCustomerOrders(PartnerCenterAuthToken saToken, DelegatingHandler handler, string resellerCid, string customerCid)
        {
            Orders orders = null;
            try
            {
                var path = string.Format("{0}/{1}/orders?recipient_customer_id={2}", Constants.CloudSolutionProviderAPIsUrl, resellerCid, customerCid);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await PartnerCenterApiInvoke(uri, saToken, "Get", handler, payload, null))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        orders = JsonConvert.DeserializeObject<Orders>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return orders;

        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="saToken"></param>
        /// <param name="handler"></param>
        /// <param name="resellerCid"></param>
        /// <param name="customerCid"></param>
        /// <returns></returns>
        public static async Task<OrderItem> GetCustomerOrderById(PartnerCenterAuthToken saToken, DelegatingHandler handler, string resellerCid, string customerCid)
        {
            OrderItem orderItem = null;
            try
            {
                var path = string.Format("{0}/{1}/orders?recipient_customer_id={2}", Constants.CloudSolutionProviderAPIsUrl, resellerCid, customerCid);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await PartnerCenterApiInvoke(uri, saToken, "Get", handler, payload, null))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        orderItem = JsonConvert.DeserializeObject<OrderItem>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return orderItem;

        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="saToken"></param>
        /// <param name="handler"></param>
        /// <param name="resellerCid"></param>
        /// <param name="newOrder"></param>
        /// <returns></returns>
        public static async Task<OrderItem> NewOrder(PartnerCenterAuthToken saToken, DelegatingHandler handler, string resellerCid, NewOrder newOrder)
        {
            OrderItem orderItem = null;
            try
            {
                var path = string.Format("{0}/{1}/orders", Constants.CloudSolutionProviderAPIsUrl, resellerCid);
                Uri uri = new Uri(path);
                HttpContent payload = new StringContent(JsonConvert.SerializeObject(newOrder), Encoding.UTF8, Constants.JsonContentType);
                using (HttpResponseMessage response = await PartnerCenterApiInvoke(uri, saToken, "Post", handler, payload, null))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        orderItem = JsonConvert.DeserializeObject<OrderItem>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return orderItem;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="saToken"></param>
        /// <param name="handler"></param>
        /// <param name="resellerCid"></param>
        /// <param name="orderId"></param>
        /// <param name="ifMatch"></param>
        /// <param name="updateOrder"></param>
        /// <returns></returns>
        public static async Task<OrderItem> UpdateOrder(PartnerCenterAuthToken saToken, DelegatingHandler handler, string resellerCid, string orderId, string ifMatch, UpdateOrder updateOrder)
        {
            OrderItem orderItem = null;
            try
            {
                var path = string.Format("{0}/{1}/orders/{2}", Constants.CloudSolutionProviderAPIsUrl, resellerCid, orderId);
                Uri uri = new Uri(path);
                HttpContent payload = new StringContent(JsonConvert.SerializeObject(updateOrder), Encoding.UTF8, Constants.JsonContentType);
                using (HttpResponseMessage response = await PartnerCenterApiInvoke(uri, saToken, "Patch", handler, payload, ifMatch))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        orderItem = JsonConvert.DeserializeObject<OrderItem>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return orderItem;
        }

        #endregion

        #region Subscriptions Management Related Methods
        /// <summary>
        /// 
        /// </summary>
        /// <param name="saToken"></param>
        /// <param name="handler"></param>
        /// <param name="resellerCid"></param>
        /// <param name="customerCid"></param>
        /// <returns></returns>
        public static async Task<Subscriptions> GetSubscriptions(PartnerCenterAuthToken saToken, DelegatingHandler handler, string resellerCid, string customerCid)
        {
            Subscriptions subscription = null;
            try
            {
                var path = string.Format("{0}/{1}/subscriptions?recipient_customer_id={2}", Constants.CloudSolutionProviderAPIsUrl, resellerCid, customerCid);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await PartnerCenterApiInvoke(uri, saToken, "Get", handler, payload, null))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        subscription = JsonConvert.DeserializeObject<Subscriptions>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return subscription;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="saToken"></param>
        /// <param name="handler"></param>
        /// <param name="resellerCid"></param>
        /// <param name="subscriptionid"></param>
        /// <returns></returns>
        public static async Task<SubscriptionItem> GetSubscriptionById(PartnerCenterAuthToken saToken, DelegatingHandler handler, string resellerCid, string subscriptionId)
        {
            SubscriptionItem subscriptionItem = null;
            try
            {
                var path = string.Format("{0}/{1}/subscriptions/{2}", Constants.CloudSolutionProviderAPIsUrl, resellerCid, subscriptionId);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await PartnerCenterApiInvoke(uri, saToken, "Get", handler, payload, null))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        subscriptionItem = JsonConvert.DeserializeObject<SubscriptionItem>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return subscriptionItem;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="saToken"></param>
        /// <param name="handler"></param>
        /// <param name="resellerCid"></param>
        /// <param name="subscriptionId"></param>
        /// <param name="suspendSubscription"></param>
        /// <returns></returns>
        public static async Task<SubscriptionItem> SuspendSubscriptionById(PartnerCenterAuthToken saToken, DelegatingHandler handler, string resellerCid, string subscriptionId, SuspendSubscription suspendSubscription)
        {
            SubscriptionItem subscriptionItem = null;
            try
            {
                var path = string.Format("{0}/{1}/subscriptions/{2}/add-suspension", Constants.CloudSolutionProviderAPIsUrl, resellerCid, subscriptionId);
                Uri uri = new Uri(path);
                HttpContent payload = new StringContent(JsonConvert.SerializeObject(suspendSubscription), Encoding.UTF8, Constants.JsonContentType);
                using (HttpResponseMessage response = await PartnerCenterApiInvoke(uri, saToken, "Post", handler, payload, null))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        subscriptionItem = JsonConvert.DeserializeObject<SubscriptionItem>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return subscriptionItem;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="saToken"></param>
        /// <param name="handler"></param>
        /// <param name="resellerCid"></param>
        /// <param name="subscriptionId"></param>
        /// <param name="reactivateSuspendedSubscription"></param>
        /// <returns></returns>
        public static async Task<SubscriptionItem> ReactivateSuspendedSubscriptionById(PartnerCenterAuthToken saToken, DelegatingHandler handler, string resellerCid, string subscriptionId, ReactivateSuspendedSubscription reactivateSuspendedSubscription)
        {
            SubscriptionItem subscriptionItem = null;
            try
            {
                var path = string.Format("{0}/{1}/subscriptions/{2}/remove-suspension", Constants.CloudSolutionProviderAPIsUrl, resellerCid, subscriptionId);
                Uri uri = new Uri(path);
                HttpContent payload = new StringContent(JsonConvert.SerializeObject(reactivateSuspendedSubscription), Encoding.UTF8, Constants.JsonContentType);
                using (HttpResponseMessage response = await PartnerCenterApiInvoke(uri, saToken, "Post", handler, payload, null))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        subscriptionItem = JsonConvert.DeserializeObject<SubscriptionItem>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return subscriptionItem;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="saToken"></param>
        /// <param name="handler"></param>
        /// <param name="resellerCid"></param>
        /// <param name="subscriptionId"></param>
        /// <param name="update"></param>
        /// <returns></returns>
        public static async Task<SubscriptionItem> UpdateSubscriptionAdvisorPartnerId(PartnerCenterAuthToken saToken, DelegatingHandler handler, string resellerCid, string subscriptionId, UpdateSubscriptionAdvisorId update)
        {
            SubscriptionItem subscriptionItem = null;
            try
            {
                var path = string.Format("{0}/{1}/subscriptions/{2}/update-advisor-partner-id", Constants.CloudSolutionProviderAPIsUrl, resellerCid, subscriptionId);
                Uri uri = new Uri(path);
                HttpContent payload = new StringContent(JsonConvert.SerializeObject(update), Encoding.UTF8, Constants.JsonContentType);
                using (HttpResponseMessage response = await PartnerCenterApiInvoke(uri, saToken, "Post", handler, payload, null))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        subscriptionItem = JsonConvert.DeserializeObject<SubscriptionItem>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return subscriptionItem;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="saToken"></param>
        /// <param name="handler"></param>
        /// <param name="resellerCid"></param>
        /// <param name="subscriptionId"></param>
        /// <param name="update"></param>
        /// <returns></returns>
        public static async Task<SubscriptionItem> UpdateSubscriptionFriendlyName(PartnerCenterAuthToken saToken, DelegatingHandler handler, string resellerCid, string subscriptionId, UpdateSubscriptionFriendlyName update)
        {
            SubscriptionItem subscriptionItem = null;
            try
            {
                var path = string.Format("{0}/{1}/subscriptions/{2}/update-friendly-name", Constants.CloudSolutionProviderAPIsUrl, resellerCid, subscriptionId);
                Uri uri = new Uri(path);
                HttpContent payload = new StringContent(JsonConvert.SerializeObject(update), Encoding.UTF8, Constants.JsonContentType);
                using (HttpResponseMessage response = await PartnerCenterApiInvoke(uri, saToken, "Post", handler, payload, null))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        subscriptionItem = JsonConvert.DeserializeObject<SubscriptionItem>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return subscriptionItem;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="saToken"></param>
        /// <param name="handler"></param>
        /// <param name="resellerCid"></param>
        /// <param name="update"></param>
        /// <returns></returns>
        public static async Task<OrderItem> TransitionSubscription(PartnerCenterAuthToken saToken, DelegatingHandler handler, string resellerCid, UpdateOrder update)
        {
            OrderItem orderItem = null;
            try
            {
                var path = string.Format("{0}/{1}/orders", Constants.CloudSolutionProviderAPIsUrl, resellerCid);
                Uri uri = new Uri(path);
                HttpContent payload = new StringContent(JsonConvert.SerializeObject(update), Encoding.UTF8, Constants.JsonContentType);
                using (HttpResponseMessage response = await PartnerCenterApiInvoke(uri, saToken, "Post", handler, payload, null))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        orderItem = JsonConvert.DeserializeObject<OrderItem>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return orderItem;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="saToken"></param>
        /// <param name="handler"></param>
        /// <param name="resellerCid"></param>
        /// <param name="orderId"></param>
        /// <param name="update"></param>
        /// <returns></returns>
        public static async Task<bool> ConvertSubscription(PartnerCenterAuthToken saToken, DelegatingHandler handler, string resellerCid, string orderId, UpdateOrder update)
        {
            try
            {
                var path = string.Format("{0}/{1}/orders/{2}", Constants.CloudSolutionProviderAPIsUrl, resellerCid, orderId);
                Uri uri = new Uri(path);
                HttpContent payload = new StringContent(JsonConvert.SerializeObject(update), Encoding.UTF8, Constants.JsonContentType);
                using (HttpResponseMessage response = await PartnerCenterApiInvoke(uri, saToken, "Patch", handler, payload, null))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        return true;
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }
        #endregion

        #region Entitlements Management Related Methods
        public static async Task<Entitlements> GetEntitlements(PartnerCenterAuthToken caToken, DelegatingHandler handler, string customerCid)
        {
            Entitlements entitlement = null;
            try
            {
                var path = string.Format("{0}/{1}/entitlements", Constants.CloudSolutionProviderAPIsUrl, customerCid);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await PartnerCenterApiInvoke(uri, caToken, "Get", handler, payload, null))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        entitlement = JsonConvert.DeserializeObject<Entitlements>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return entitlement;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="caToken"></param>
        /// <param name="handler"></param>
        /// <param name="customerCid"></param>
        /// <param name="entitlementId"></param>
        /// <returns></returns>
        public static async Task<EntitlementItem> GetEntitlementById(PartnerCenterAuthToken caToken, DelegatingHandler handler, string customerCid, string entitlementId)
        {
            EntitlementItem entitlementItem = null;
            try
            {
                var path = string.Format("{0}/{1}/entitlements/{2}", Constants.CloudSolutionProviderAPIsUrl, customerCid, entitlementId);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await PartnerCenterApiInvoke(uri, caToken, "Get", handler, payload, null))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        entitlementItem = JsonConvert.DeserializeObject<EntitlementItem>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return entitlementItem;
        }
        #endregion

        #region Profiles Management Related Methods
        /// <summary>
        /// 
        /// </summary>
        /// <param name="caToken"></param>
        /// <param name="handler"></param>
        /// <param name="customerId"></param>
        /// <returns></returns>
        public static async Task<Profiles> GetProfiles(PartnerCenterAuthToken caToken, DelegatingHandler handler, string customerId)
        {
            Profiles profile = null;
            try
            {
                var path = string.Format("{0}/{1}/profiles", Constants.CloudSolutionProviderAPIsUrl, customerId);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await PartnerCenterApiInvoke(uri, caToken, "Get", handler, payload, null))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        profile = JsonConvert.DeserializeObject<Profiles>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return profile;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="caToken"></param>
        /// <param name="handler"></param>
        /// <param name="customerId"></param>
        /// <param name="profileId"></param>
        /// <param name="updateProfile"></param>
        /// <returns></returns>
        public static async Task<ProfileItem> UpdateProfile(PartnerCenterAuthToken caToken, DelegatingHandler handler, string customerId, string profileId, UpdateCustomerProfile updateProfile)
        {
            ProfileItem profileItem = null;
            try
            {
                var path = string.Format("{0}/{1}/profiles/{2}", Constants.CloudSolutionProviderAPIsUrl, customerId, profileId);
                Uri uri = new Uri(path);
                HttpContent payload = new StringContent(JsonConvert.SerializeObject(updateProfile), Encoding.UTF8, Constants.JsonContentType);
                using (HttpResponseMessage response = await PartnerCenterApiInvoke(uri, caToken, "Post", handler, payload, null))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        profileItem = JsonConvert.DeserializeObject<ProfileItem>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return profileItem;
        }
        #endregion

        #region Events Management Related Methods
        public static async Task<EventStream> CreateEventStream(PartnerCenterAuthToken saToken, DelegatingHandler handler, string resellerCid, string streamName)
        {
            EventStream stream = null;
            try
            {
                var path = string.Format("{0}/{1}/subscription-streams/{2}/{3}", Constants.CloudSolutionProviderAPIsUrl, resellerCid, resellerCid, streamName);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await PartnerCenterApiInvoke(uri, saToken, "Put", handler, payload, null))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        stream = JsonConvert.DeserializeObject<EventStream>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return stream;
        }

        public static async Task<EventStream> DeleteEventStream(PartnerCenterAuthToken saToken, DelegatingHandler handler, string resellerCid, string streamName)
        {
            EventStream stream = null;
            try
            {
                var path = string.Format("{0}/{1}/subscription-streams/{2}/{3}", Constants.CloudSolutionProviderAPIsUrl, resellerCid, resellerCid, streamName);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await PartnerCenterApiInvoke(uri, saToken, "Delete", handler, payload, null))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        stream = JsonConvert.DeserializeObject<EventStream>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return stream;
        }

        public static async Task<EventStreamPage> GetEventStreamPage(PartnerCenterAuthToken saToken, DelegatingHandler handler, string resellerCid, string streamName)
        {
            EventStreamPage streamPage = null;
            try
            {
                var path = string.Format("{0}/{1}/subscription-streams/{2}/{3}/pages", Constants.CloudSolutionProviderAPIsUrl, resellerCid, resellerCid, streamName);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await PartnerCenterApiInvoke(uri, saToken, "Get", handler, payload, null))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        streamPage = JsonConvert.DeserializeObject<EventStreamPage>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return streamPage;
        }
        #endregion

        #region Usage Records Management Related Methods
        public static async Task<UsageRecord> GetusageRecords(PartnerCenterAuthToken saToken, DelegatingHandler handler, string resellerCid, string entitlementId, string queryString)
        {
            UsageRecord usage = null;
            try
            {
                var path = string.Format("{0}/{1}/usage-records?entitlement_id={2}{3}", Constants.CloudSolutionProviderAPIsUrl, resellerCid, entitlementId, queryString);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await PartnerCenterApiInvoke(uri, saToken, "Get", handler, payload, null))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        usage = JsonConvert.DeserializeObject<UsageRecord>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return usage;
        }
        #endregion

        /// <summary>
        /// Generic method for invoking partner center Restful APIs
        /// </summary>
        /// <param name="uri"></param>
        /// <param name="authToken"></param>
        /// <param name="verb"></param>
        /// <param name="handler"></param>
        /// <param name="content"></param>
        /// <param name="ifMatch"></param>
        /// <returns></returns>
        public static async Task<HttpResponseMessage> PartnerCenterApiInvoke(Uri uri, PartnerCenterAuthToken authToken, string verb, DelegatingHandler handler, HttpContent content, string ifMatch)
        {
            using (var client = new HttpClient(handler))
            {
                HttpResponseMessage response = null;
                client.DefaultRequestHeaders.Add("Authorization", String.Format("Bearer {0}", authToken.AccessToken));
                client.DefaultRequestHeaders.Add("Accept", Constants.JsonContentType);
                client.DefaultRequestHeaders.Add("api-version", Constants.CloudSolutionProviderAPIVersion);
                client.DefaultRequestHeaders.Add("x-ms-correlation-id", Guid.NewGuid().ToString());
                client.DefaultRequestHeaders.Add("x-ms-tracking-id", Guid.NewGuid().ToString());
                if(!string.IsNullOrEmpty(ifMatch))
                    client.DefaultRequestHeaders.Add("If-Match", ifMatch);
                if (String.Equals(verb, "get", StringComparison.OrdinalIgnoreCase))
                {
                    response = await client.GetAsync(uri);
                }
                else if (String.Equals(verb, "delete", StringComparison.OrdinalIgnoreCase))
                {
                    response = await client.DeleteAsync(uri);
                }
                else if (String.Equals(verb, "post", StringComparison.OrdinalIgnoreCase))
                {
                    response = await client.PostAsync(uri, content);
                }
                else if (String.Equals(verb, "put", StringComparison.OrdinalIgnoreCase))
                {
                    response = await client.PutAsync(uri, content);
                }
                else if (String.Equals(verb, "patch", StringComparison.OrdinalIgnoreCase))
                {
                    using (var message = new HttpRequestMessage(new HttpMethod("PATCH"), uri))
                    {
                        message.Content = content;
                        response = await client.SendAsync(message).ConfigureAwait(false);
                    }
                }
                else
                {
                    throw new InvalidOperationException(String.Format("Invalid http verb {0}!", verb));
                }
                return response;
            }
        }
    }
}
