﻿using Caliburn.Micro;
using Microsoft.Samples.CloudSolutionProviderControlPanel.Models.Events;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace Microsoft.Samples.CloudSolutionProviderControlPanel.Utility
{
    public class HttpLoggingHandler : DelegatingHandler
    {
        private readonly bool verbose;
        private readonly IEventAggregator eventAg;

        public HttpLoggingHandler(IEventAggregator eventAggregator, HttpMessageHandler innerHandler, bool isVerbose) : base (innerHandler)
        {
            verbose = isVerbose;
            eventAg = eventAggregator;
        }

        protected override async Task<HttpResponseMessage> SendAsync(HttpRequestMessage request, CancellationToken cancellationToken)
        {
            if (verbose)
            {
                LogInfoLn(eventAg, "---------- Request -----------------------");

                LogInfoLn(eventAg, "{0} {1} HTTP/{2}", request.Method, request.RequestUri.PathAndQuery, request.Version);
                LogInfoLn(eventAg, "Host: {0}", request.RequestUri.Host);

                foreach (var header in request.Headers)
                {
                    string headerVal = string.Empty;
                    if (String.Equals("Authorization", header.Key))
                    {
                        headerVal = header.Value.First().Substring(0, 70) + "...";
                    }
                    else
                    {
                        headerVal = String.Join("; ", header.Value);
                    }
                    LogInfoLn(eventAg, "{0}: {1}", header.Key, headerVal);
                }

                await DumpContent(request.Content);
            }

            var watch = new Stopwatch();
            watch.Start();
            HttpResponseMessage response = await base.SendAsync(request, cancellationToken);
            watch.Stop();

            int responseStatusCocde = (int)response.StatusCode;

            if (verbose)
            {
                LogInfoLn(eventAg, "---------- Response ({0} ms) ------------", watch.ElapsedMilliseconds);
                LogInfoLn(eventAg, "HTTP/{0} {1} {2}", response.Version, responseStatusCocde, response.StatusCode);
                foreach (var header in response.Headers)
                {
                    LogInfoLn(eventAg, "{0}: {1}", header.Key, String.Join("; ", header.Value));
                }
            }

            await DumpContent(response.Content, isErrorRequest(responseStatusCocde));
           
            return response;
        }

        private async Task DumpContent(HttpContent content, bool isErrorContent = false)
        {
            if (content == null || content.Headers.ContentType == null)
            {
                return;
            }
            var result = await content.ReadAsStringAsync();

            LogInfoLn(eventAg, string.Empty);
            if (!string.IsNullOrWhiteSpace(result))
            {
                //dynamic parsedJson = JsonConvert.DeserializeObject(result);
                //string indentedResult = JsonConvert.SerializeObject(parsedJson, Formatting.Indented);

                TraceEvent traceMessage = new TraceEvent(isErrorContent ? Models.Events.TraceEventType.Error : Models.Events.TraceEventType.Info, result);
                
                Logger.WriteLine(eventAg, traceMessage);
            }
        }

        private bool isErrorRequest(int responseStatusCode)
        {
            return responseStatusCode >= 400 && responseStatusCode < 600;
        }

        private static void LogInfoLn(IEventAggregator eventAggregator, string format, params object[] args)
        {
            TraceEvent traceMessage = new TraceEvent(Models.Events.TraceEventType.Info, string.Format(format, args));

            Logger.WriteLine(eventAggregator, traceMessage);
        }
    }
}
