﻿using Caliburn.Micro;
using Microsoft.Samples.AuthenticationManagement;
using Microsoft.Samples.CloudSolutionProviderControlPanel.Utility;
using Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.Azure.ResourceManagement;
using Microsoft.Samples.CloudSolutionProviderManagement.Azure;
using Microsoft.Samples.CloudSolutionProviderManagement.Azure.DataModel.ResourceManager;
using Microsoft.Samples.CloudSolutionProviderManagement.AzureActiveDirectory;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.Composition;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.Azure
{
    [Export(typeof(AzureManagementViewModel))]
    public class AzureManagementViewModel : Conductor<object>
    {
        private readonly IWindowManager windowManager;
        private readonly IEventAggregator eventAggregator;
        private string tenantId;
        private string subscriptionGuid;
        private LoginType authType;
        private readonly BackgroundWorker worker = new BackgroundWorker();
        private Visibility showLoadIndicator = Visibility.Collapsed;
        private AzureSubscription subscription = null;

        [ImportingConstructor]
        public AzureManagementViewModel(IWindowManager winMgr, IEventAggregator evt, string tId, string subscriptionId, LoginType loginType)
        {
            windowManager = winMgr;
            eventAggregator = evt;
            tenantId = tId;
            subscriptionGuid = subscriptionId;
            authType = loginType;
            worker.DoWork += Worker_DoWork;
            worker.RunWorkerCompleted += Worker_RunWorkerCompleted;
            this.DisplayName = "Manage Azure Subscription: " + subscriptionGuid;
            Login();
            worker.RunWorkerAsync();
            ShowLoadIndicator = Visibility.Visible;
            
        }

        public async void Login()
        {
            if (authType == LoginType.Interactive)
            {
                await App.CurrentAppContext.AuthManager.AcquireTokens(Constants.ClientId, tenantId, "https://management.azure.com/");
            }
        }

        void Worker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            ShowLoadIndicator = Visibility.Collapsed;
            Refresh();
            ActivateItem(new ResourceGroupListViewModel(windowManager, eventAggregator, tenantId, subscriptionGuid, authType));
        }

        void Worker_DoWork(object sender, DoWorkEventArgs e)
        {
            var handler = new HttpLoggingHandler(eventAggregator, new HttpClientHandler(), true);
            AuthTokenCacheInfo cacheInfo = null;
            if (authType == LoginType.Interactive)
            {
                Task<AuthTokenCacheInfo> cacheInfoTask = App.CurrentAppContext.AuthManager.GetToken(Constants.ClientId, tenantId, "https://management.azure.com/");
                cacheInfoTask.Wait();
                cacheInfo = cacheInfoTask.Result;
            }
            else if (authType == LoginType.Spn)
            {
                cacheInfo = App.CurrentAppContext.AuthManager.GetTokenBySpn(tenantId, App.CurrentAppContext.Settings.ManagementAppId, App.CurrentAppContext.Settings.ManagementAppSecret, "https://management.azure.com/");
            }
            if (cacheInfo != null)
            {

                Task<AzureSubscription> subTask = AzureResourceManager.GetAzureSubscription(cacheInfo, handler, subscriptionGuid);
                subTask.Wait();
                subscription = subTask.Result;
            }
        }

        public Visibility ShowLoadIndicator
        {
            get
            {
                return showLoadIndicator;
            }
            set
            {
                showLoadIndicator = value;
                NotifyOfPropertyChange(() => ShowLoadIndicator);
            }
        }

        public string SubscriptionId
        {
            get {
                if (subscription != null)
                {
                    return subscription.SubscriptionId;
                }
                else
                    return string.Empty;
            }
        }

        public string SubscriptionName
        {
            get
            {
                if (subscription != null)
                {
                    return subscription.DisplayName;
                }
                else
                    return string.Empty;
            }
        }

        public string State
        {
            get
            {
                if (subscription != null)
                {
                    return subscription.State;
                }
                else
                    return string.Empty;
            }
        }

        /// <summary>
        /// check login type to determine we can create the azure role assignment or not
        /// </summary>
        public bool CanAssign
        {
            get
            {
                return authType == LoginType.Interactive;
            }
        }

        //public async void AssignAdminRole()
        //{
        //    MessageBoxResult result = MessageBox.Show("Are you sure you like to grant admin role of this subscription to your application specified in configuration?", "Please Confirm", MessageBoxButton.YesNoCancel, MessageBoxImage.Exclamation);
        //    if (result == MessageBoxResult.Yes)
        //    {

        //    }
        //}
    }
}
