﻿using Caliburn.Micro;
using Microsoft.Samples.AuthenticationManagement;
using Microsoft.Samples.CloudSolutionProviderControlPanel.Utility;
using Microsoft.Samples.CloudSolutionProviderManagement.Azure;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.Composition;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.Azure.VirtualMachine
{
    [Export(typeof(VirtualMachineViewModel))]
    public class VirtualMachineViewModel : Conductor<object>
    {
        private readonly IWindowManager windowManager;
        private readonly IEventAggregator eventAggregator;
        private string tenantId;
        private string subscriptionId;
        private string resourceGroupName;
        private CloudSolutionProviderManagement.Azure.DataModel.Compute.VirtualMachine vmItem;
        private LoginType authType;
        private readonly BackgroundWorker worker = new BackgroundWorker();

        [ImportingConstructor]
        public VirtualMachineViewModel(IWindowManager winMgr, IEventAggregator evt, string tId, string sId, string rgName, CloudSolutionProviderManagement.Azure.DataModel.Compute.VirtualMachine item, LoginType loginType)
        {
            windowManager = winMgr;
            eventAggregator = evt;
            tenantId = tId;
            subscriptionId = sId;
            resourceGroupName = rgName;
            vmItem = item;
            authType = loginType;
        }

        public string VirtualMachineId
        {
            get
            {
                return vmItem.Id;
            }
        }

        public string VirtualMachineName
        {
            get
            {
                return vmItem.Name;
            }
        }

        public string Location
        {
            get
            {
                return vmItem.Location;
            }
        }

        public string ProvisioningState
        {
            get
            {
                return vmItem.Properties.ProvisioningState;
            }
        }
        

        public void HardwareProfile()
        {
            ActivateItem(new HardwareProfileViewModel(windowManager, eventAggregator, tenantId, subscriptionId, resourceGroupName, vmItem, authType));
        }

        public void OSProfile()
        {
            ActivateItem(new OSProfileViewModel(windowManager, eventAggregator, tenantId, subscriptionId, resourceGroupName, vmItem, authType));
        }

        public void NetworkProfile()
        {
            ActivateItem(new NetworkProfileViewModel(windowManager, eventAggregator, tenantId, subscriptionId, resourceGroupName, vmItem, authType));
        }

        public void StorageProfile()
        {
            ActivateItem(new StorageProfileViewModel(windowManager, eventAggregator, tenantId, subscriptionId, resourceGroupName, vmItem, authType));
        }

        public void Extensions()
        {
            ActivateItem(new VirtualMachineExtensionListViewModel(windowManager, eventAggregator, tenantId, subscriptionId, resourceGroupName, vmItem, authType));
        }

        public async void StartVM()
        {
            MessageBoxResult result = MessageBox.Show("Are you sure you like to start virtual machine?", "Please Confirm", MessageBoxButton.YesNoCancel, MessageBoxImage.Exclamation);
            if (result == MessageBoxResult.Yes)
            {
                try
                {
                    var handler = new HttpLoggingHandler(eventAggregator, new HttpClientHandler(), true);
                    AuthTokenCacheInfo cacheInfo = null;
                    if (authType == LoginType.Interactive)
                    {
                        Task<AuthTokenCacheInfo> cacheInfoTask = App.CurrentAppContext.AuthManager.GetToken(Constants.ClientId, tenantId, "https://management.azure.com/");
                        cacheInfoTask.Wait();
                        cacheInfo = cacheInfoTask.Result;
                    }
                    else if (authType == LoginType.Spn)
                    {
                        cacheInfo = App.CurrentAppContext.AuthManager.GetTokenBySpn(tenantId, App.CurrentAppContext.Settings.ManagementAppId, App.CurrentAppContext.Settings.ManagementAppSecret, "https://management.azure.com/");
                    }
                    if (cacheInfo != null)
                    {
                        string operationStatusLink = await AzureResourceManager.StartVirtualMachine(cacheInfo, handler, subscriptionId, resourceGroupName, vmItem.Name);
                        
                    }
                }
                catch(Exception ex)
                {
                    Logger.ErrorLine(eventAggregator, "{0} {1}", ex.Message, ex.StackTrace);
                    MessageBox.Show(ex.Message, "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                }
                
            }
        }

        public async void StopVM()
        {
            MessageBoxResult result = MessageBox.Show("Are you sure you like to stop virtual machine?", "Please Confirm", MessageBoxButton.YesNoCancel, MessageBoxImage.Exclamation);
            if (result == MessageBoxResult.Yes)
            {
                try
                {
                    var handler = new HttpLoggingHandler(eventAggregator, new HttpClientHandler(), true);
                    AuthTokenCacheInfo cacheInfo = null;
                    if (authType == LoginType.Interactive)
                    {
                        Task<AuthTokenCacheInfo> cacheInfoTask = App.CurrentAppContext.AuthManager.GetToken(Constants.ClientId, tenantId, "https://management.azure.com/");
                        cacheInfoTask.Wait();
                        cacheInfo = cacheInfoTask.Result;
                    }
                    else if (authType == LoginType.Spn)
                    {
                        cacheInfo = App.CurrentAppContext.AuthManager.GetTokenBySpn(tenantId, App.CurrentAppContext.Settings.ManagementAppId, App.CurrentAppContext.Settings.ManagementAppSecret, "https://management.azure.com/");
                    }
                    if (cacheInfo != null)
                    {
                        string operationStatusLink = await AzureResourceManager.StopVirtualMachine(cacheInfo, handler, subscriptionId, resourceGroupName, vmItem.Name);

                    }
                }
                catch (Exception ex)
                {
                    Logger.ErrorLine(eventAggregator, "{0} {1}", ex.Message, ex.StackTrace);
                    MessageBox.Show(ex.Message, "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                }

            }
        }

        public async void DeallocateVM()
        {
            MessageBoxResult result = MessageBox.Show("Are you sure you like to stop and deallocate virtual machine?", "Please Confirm", MessageBoxButton.YesNoCancel, MessageBoxImage.Exclamation);
            if (result == MessageBoxResult.Yes)
            {
                try
                {
                    var handler = new HttpLoggingHandler(eventAggregator, new HttpClientHandler(), true);
                    AuthTokenCacheInfo cacheInfo = null;
                    if (authType == LoginType.Interactive)
                    {
                        Task<AuthTokenCacheInfo> cacheInfoTask = App.CurrentAppContext.AuthManager.GetToken(Constants.ClientId, tenantId, "https://management.azure.com/");
                        cacheInfoTask.Wait();
                        cacheInfo = cacheInfoTask.Result;
                    }
                    else if (authType == LoginType.Spn)
                    {
                        cacheInfo = App.CurrentAppContext.AuthManager.GetTokenBySpn(tenantId, App.CurrentAppContext.Settings.ManagementAppId, App.CurrentAppContext.Settings.ManagementAppSecret, "https://management.azure.com/");
                    }
                    if (cacheInfo != null)
                    {
                        string operationStatusLink = await AzureResourceManager.DeallocateVirtualMachine(cacheInfo, handler, subscriptionId, resourceGroupName, vmItem.Name);

                    }
                }
                catch (Exception ex)
                {
                    Logger.ErrorLine(eventAggregator, "{0} {1}", ex.Message, ex.StackTrace);
                    MessageBox.Show(ex.Message, "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                }

            }
        }
    }
}
