﻿using Caliburn.Micro;
using Microsoft.Samples.AuthenticationManagement;
using Microsoft.Samples.CloudSolutionProviderControlPanel.Utility;
using Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.Azure;
using Microsoft.Samples.CloudSolutionProviderManagement.PartnerCenter;
using Microsoft.Samples.CloudSolutionProviderManagement.PartnerCenter.DataModel;
using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.PartnerCenter.Subscription
{
    [Export(typeof(SubscriptionListViewModel))]
    public class SubscriptionViewModel : Screen
    {
        private readonly IWindowManager windowManager;
        private readonly IEventAggregator eventAggregator;
        private SubscriptionItem subItem;
        private string customerTId;
        private LoginType authType;

        [ImportingConstructor]
        public SubscriptionViewModel(IWindowManager winMgr, IEventAggregator evt, string customerTenantId, SubscriptionItem item, LoginType loginType)
        {
            windowManager = winMgr;
            eventAggregator = evt;
            customerTId = customerTenantId;
            subItem = item;
            authType = loginType;
        }

        public string SubscriptionId
        {
            get
            {
                return subItem.Id;
            }
        }

        public string OrderId
        {
            get
            {
                return subItem.OrderId;
            }
        }

        public string CreationDate
        {
            get
            {
                return subItem.CreationDate;
            }
        }

        public string EffectiveStartDate
        {
            get
            {
                return subItem.EffectiveStartDate;
            }
        }

        public string CommitmentEndDate
        {
            get
            {
                return subItem.CommitmentEndDate;
            }
        }

        public int Quantity
        {
            get
            {
                return subItem.Quantity;
            }
        }

        public string State
        {
            get
            {
                return subItem.State;
            }
        }

        public string ETag
        {
            get
            {
                return subItem.Etag;
            }
        }

        public string SuspensionReasons
        {
            get
            {
                string reasons = string.Empty;
                foreach (object o in subItem.SuspensionReasons)
                {
                    reasons += o.ToString() + ",";
                }
                return reasons;
            }
        }

        public string Offer
        {
            get
            {
                return subItem.OfferUri;
            }
        }

        public bool IsAzureSubscription
        {
            get
            {
                return subItem.OfferUri.ToLower().Contains("ms-azr");
            }
        }

        public void UpdateFriendlyName()
        {
            windowManager.ShowDialog(new UpdateSubscriptionFriendlyNameViewModel(eventAggregator, App.CurrentAppContext.Settings.ResellerTenantId, subItem.Id));
        }

        
        
        public async void SuspendSubscription()
        {
            MessageBoxResult result = MessageBox.Show("Are you sure you like to suspend this subscription?", "Please Confirm", MessageBoxButton.YesNoCancel, MessageBoxImage.Exclamation);
            if (result == MessageBoxResult.Yes)
            {
                try
                {
                    SuspendSubscription suspension = new CloudSolutionProviderManagement.PartnerCenter.DataModel.SuspendSubscription();
                    suspension.SuspensionReason = "CustomerCancellation";
                    suspension.SuspensionReasonComment = "customer cancellation is the only supported reason";
                    var handler = new HttpLoggingHandler(eventAggregator, new HttpClientHandler(), true);
                    AuthTokenCacheInfo cacheInfo = App.CurrentAppContext.AuthManager.GetTokenBySpn(App.CurrentAppContext.Settings.ResellerTenantId, App.CurrentAppContext.Settings.PartnerCenterAppId, App.CurrentAppContext.Settings.PartnerCenterAppSecret, Microsoft.Samples.CloudSolutionProviderManagement.Constants.AzureADGraphAPIUrl);
                    PartnerCenterAuthToken authToken = new PartnerCenterAuthToken(cacheInfo.AccessToken, cacheInfo.ExpiresOn);

                    PartnerCenterAuthToken saToken = await PartnerCenterManager.GetSalesAgentToken(authToken, handler);

                    handler = new HttpLoggingHandler(eventAggregator, new HttpClientHandler(), true);
                    CloudSolutionProviderManagement.PartnerCenter.DataModel.SubscriptionItem item = await PartnerCenterManager.SuspendSubscriptionById(saToken, handler, App.CurrentAppContext.Settings.ResellerTenantId, subItem.Id, suspension);
                    
                }
                catch (Exception ex)
                {
                    Logger.ErrorLine(eventAggregator, "{0} {1}", ex.Message, ex.StackTrace);
                    MessageBox.Show(ex.Message, "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                }
            }
        }

        public async void RemoveSuspension()
        {
            MessageBoxResult result = MessageBox.Show("Are you sure you like to remove suspension for this subscription?", "Please Confirm", MessageBoxButton.YesNoCancel, MessageBoxImage.Exclamation);
            if (result == MessageBoxResult.Yes)
            {
                try
                {
                    ReactivateSuspendedSubscription suspension = new CloudSolutionProviderManagement.PartnerCenter.DataModel.ReactivateSuspendedSubscription();
                    suspension.SuspensionReason = "CustomerCancellation";
                    var handler = new HttpLoggingHandler(eventAggregator, new HttpClientHandler(), true);
                    AuthTokenCacheInfo cacheInfo = App.CurrentAppContext.AuthManager.GetTokenBySpn(App.CurrentAppContext.Settings.ResellerTenantId, App.CurrentAppContext.Settings.PartnerCenterAppId, App.CurrentAppContext.Settings.PartnerCenterAppSecret, Microsoft.Samples.CloudSolutionProviderManagement.Constants.AzureADGraphAPIUrl);
                    PartnerCenterAuthToken authToken = new PartnerCenterAuthToken(cacheInfo.AccessToken, cacheInfo.ExpiresOn);

                    PartnerCenterAuthToken saToken = await PartnerCenterManager.GetSalesAgentToken(authToken, handler);

                    handler = new HttpLoggingHandler(eventAggregator, new HttpClientHandler(), true);
                    CloudSolutionProviderManagement.PartnerCenter.DataModel.SubscriptionItem item = await PartnerCenterManager.ReactivateSuspendedSubscriptionById(saToken, handler, App.CurrentAppContext.Settings.ResellerTenantId, subItem.Id, suspension);

                }
                catch (Exception ex)
                {
                    Logger.ErrorLine(eventAggregator, "{0} {1}", ex.Message, ex.StackTrace);
                    MessageBox.Show(ex.Message, "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                }
            }
        }

        public void ManageAzure()
        {
            windowManager.ShowWindow(new AzureManagementViewModel(windowManager, eventAggregator, customerTId, subItem.Id, authType));
        }
    }
}
