﻿param(
    [string] $UserName = "",
    [string] $Password = "",
    [string] $ResourceGroupName = "customerDemoSiteRecoveryRG",
    [string] $SubscriptionName = "Microsoft Azure",
    [string] $VaultName = "customerDemoSiteRecoveryVault",
	[string] $VaultGeo  = "West US",
	[string] $OutputPathForSettingsFile = "D:\vaults",
    [string] $PrimaryCloudName = "California_Cloud",
    [string] $RecoveryCloudName = "Seattle_Cloud",
    [string] $ProtectionProfileName = "wpcProtectionProfile5",
    [string] $VMName = "SQLVM",                                # Other VMs on Primary Cloud: {tfoVM, APPVM, SQLVMBlank}
    [int] $JobQueryWaitTimeInSeconds = 60
	)

Function global:WaitForJobCompletion
{ 
	param(
        [string] $JobId,
        [int] $JobQueryWaitTimeInSeconds = 60,
        [string] $Message = "NA"
        )
        $isJobLeftForProcessing = $true;
        do
        {
            $Job = Get-AzureSiteRecoveryJob -Id $JobId
            Write-Host $("Job Status:") -ForegroundColor Green
            $Job

            if($Job.State -eq "InProgress")
            {
	            $isJobLeftForProcessing = $true
            }
            else
            {
                $isJobLeftForProcessing = $false
            }

            if($isJobLeftForProcessing)
	        {
                if($Message -ne "NA")
                {
                    Write-Host $Message -ForegroundColor Yellow
                }
                else
                {
                    Write-Host $($($Job.ScenarioName) + " in Progress...") -ForegroundColor Yellow
                }
		        Write-Host $("Waiting for: " + $JobQueryWaitTimeInSeconds.ToString() + " Seconds") -ForegroundColor Yellow
		        Start-Sleep -Seconds $JobQueryWaitTimeInSeconds
	        }
        }While($isJobLeftForProcessing)
}

Function global:WaitForIRCompletion
{ 
	param(
        [PSObject] $VM,
        [int] $JobQueryWaitTimeInSeconds = 60
        )
        $isProcessingLeft = $true
        $IRjobs = $null

        Write-Host $("IR in Progress...") -ForegroundColor Yellow
        do
        {
            $IRjobs = Get-AzureSiteRecoveryJob -TargetObjectId $VM.Name | Sort-Object StartTime -Descending | select -First 2 | Where-Object{$_.ScenarioName -eq "PrimaryIrCompletion" -or $_.ScenarioName -eq "SecondaryIrCompletion"}
            if($IRjobs -eq $null -or $IRjobs.Count -ne 2)
            {
	            $isProcessingLeft = $true
            }
            else
            {
                $isProcessingLeft = $false
            }

            if($isProcessingLeft)
	        {
                Write-Host $("IR in Progress...") -ForegroundColor Yellow
		        Write-Host $("Waiting for: " + $JobQueryWaitTimeInSeconds.ToString() + " Seconds") -ForegroundColor Yellow
		        Start-Sleep -Seconds $JobQueryWaitTimeInSeconds
	        }
        }While($isProcessingLeft)

        Write-Host $("Finalize IR jobs:") -ForegroundColor Green
        $IRjobs
        WaitForJobCompletion -JobId $IRjobs[0].Name -JobQueryWaitTimeInSeconds $JobQueryWaitTimeInSeconds -Message $("Finalize IR in Progress...")
        WaitForJobCompletion -JobId $IRjobs[1].Name -JobQueryWaitTimeInSeconds $JobQueryWaitTimeInSeconds -Message $("Finalize IR in Progress...")
}

# Switch Azure Mode to Azure Resource Manager
Switch-AzureMode AzureResourceManager
Write-Host $("****************Azure Mode Set to Azure Resource Manager****************") -ForegroundColor Green

# Add Azure Account
$SecurePassword = ConvertTo-SecureString -AsPlainText $Password -Force
$AzureOrgIdCredential = New-Object System.Management.Automation.PSCredential -ArgumentList $UserName, $securePassword
Add-AzureAccount -Tenant "azurecustomer08.onmicrosoft.com" -Credential $AzureOrgIdCredential
Write-Host $("****************Azure Account Set****************") -ForegroundColor Green

# Set Subscription Context
Select-AzureSubscription -SubscriptionName $SubscriptionName
Write-Host $("****************Selected Subscription: " + $SubscriptionName + "****************")  -ForegroundColor Green
Get-AzureSubscription -Current

# Set Vault Context
$Vault = Get-AzureSiteRecoveryVault -ResouceGroupName $ResourceGroupName | where { $_.Name -eq $VaultName}
$VaultSetingsFile = Get-AzureSiteRecoveryVaultSettingsFile -Vault $Vault -Path $OutputPathForSettingsFile
Write-Host $("****************Vault setting file path: " + $VaultSetingsFile.FilePath + "****************") -ForegroundColor Green

Import-AzureSiteRecoveryVaultSettingsFile -Path $VaultSetingsFile.FilePath -ErrorAction Stop
Write-Host $("****************Vault Context set for vault: " + $VaultName + "****************") -ForegroundColor Green

# Enumerate registered servers
Get-AzureSiteRecoveryServer
Write-Host $("****************Fetched registered servers to vault****************") -ForegroundColor Green

# Get Protection containers
$ProtectionContainers = Get-AzureSiteRecoveryProtectionContainer
$PrimaryContainer = $ProtectionContainers | where { $_.FriendlyName -eq $PrimaryCloudName }
Write-Host $("Primary Cloud:") -ForegroundColor Green
$PrimaryContainer
$RecoveryContainer = $ProtectionContainers | where { $_.FriendlyName-eq $RecoveryCloudName }
Write-Host $("Recovery Cloud:") -ForegroundColor Green
$RecoveryContainer
Write-Host $("****************Clouds fetched before Pairing****************") -ForegroundColor Green

# Create Protection Profile
Write-Host $("Creating Protection Profile") -ForegroundColor Green
New-AzureSiteRecoveryProtectionProfile -Name $ProtectionProfileName -ReplicationProvider HyperVReplica -ReplicationMethod Online -ReplicationFrequencyInSeconds 30 -RecoveryPoints 1 -ApplicationConsistentSnapshotFrequencyInHours 0 -ReplicationPort 8083 -Authentication Kerberos -AllowReplicaDeletion 
$ProtectionProfile = Get-AzureSiteRecoveryProtectionProfile -Name $ProtectionProfileName
$ProtectionProfile
Write-Host $("****************Created Protection Profile****************") -ForegroundColor Green

# Cloud Pairing (Associate Cloud with Protection Profile)
Write-Host $("Started Cloud Pairing") -ForegroundColor Green
$currentJob = Start-AzureSiteRecoveryProtectionProfileAssociationJob -ProtectionProfile $ProtectionProfile -PrimaryProtectionContainer $PrimaryContainer -RecoveryProtectionContainer $RecoveryContainer
$currentJob
WaitForJobCompletion -JobId $currentJob.Name -JobQueryWaitTimeInSeconds $JobQueryWaitTimeInSeconds
Write-Host $("****************Clouds paired****************") -ForegroundColor Green

# Update Protection container objects after cloud pairing
$ProtectionContainers = Get-AzureSiteRecoveryProtectionContainer
$PrimaryContainer = $ProtectionContainers | where { $_.FriendlyName -eq $PrimaryCloudName }
Write-Host $("Primary Cloud:") -ForegroundColor Green
$PrimaryContainer
$RecoveryContainer = $ProtectionContainers | where { $_.FriendlyName-eq $RecoveryCloudName }
Write-Host $("Recovery Cloud:") -ForegroundColor Green
$RecoveryContainer
Write-Host $("****************Clouds fetched after Pairing****************") -ForegroundColor Green

# Get VM to enable protection
$VM = Get-AzureSiteRecoveryProtectionEntity -ProtectionContainer $PrimaryContainer | where { $_.FriendlyName -eq $VMName }
Write-Host $("VM:") -ForegroundColor Green
$VM
Write-Host $("****************VM fetched for Enabling Protection****************") -ForegroundColor Green

# Enable Protection
Write-Host $("Started Enable Protection") -ForegroundColor Green
$currentJob = Set-AzureSiteRecoveryProtectionEntity -ProtectionEntity $VM -Protection Enable
$currentJob
WaitForJobCompletion -JobId $currentJob.Name -JobQueryWaitTimeInSeconds $JobQueryWaitTimeInSeconds
WaitForIRCompletion -VM $VM -JobQueryWaitTimeInSeconds $JobQueryWaitTimeInSeconds
Write-Host $("****************Protection Enabled for VM****************") -ForegroundColor Green

# Get protected VM
$VM = Get-AzureSiteRecoveryProtectionEntity -ProtectionContainer $PrimaryContainer | where { $_.FriendlyName -eq $VMName }
Write-Host $("VM:") -ForegroundColor Green
$VM
Write-Host $("****************VM fetched after Enabling Protection****************") -ForegroundColor Green

# Start TestFailover
Write-Host $("Triggered TFO") -ForegroundColor Green
$currentJob = Start-AzureSiteRecoveryTestFailoverJob -ProtectionEntity $VM -Direction PrimaryToRecovery
$currentJob
WaitForJobCompletion -JobId $currentJob.Name -JobQueryWaitTimeInSeconds $JobQueryWaitTimeInSeconds
Write-Host $("****************TFO Started for VM****************") -ForegroundColor Green

# Complete TestFailover
Write-Host $("Started TFO Completion") -ForegroundColor Green
$currentJob = Resume-AzureSiteRecoveryJob -Id $currentJob.Name
$currentJob
WaitForJobCompletion -JobId $currentJob.Name -JobQueryWaitTimeInSeconds $JobQueryWaitTimeInSeconds
Write-Host $("****************TFO Completed for VM****************") -ForegroundColor Green

# Disable Protection
Write-Host $("Started Disable Protection") -ForegroundColor Green
$currentJob = Set-AzureSiteRecoveryProtectionEntity -ProtectionEntity $VM -Protection Disable
$currentJob
WaitForJobCompletion -JobId $currentJob.Name -JobQueryWaitTimeInSeconds $JobQueryWaitTimeInSeconds
Write-Host $("****************Disabled Protection for VM****************") -ForegroundColor Green

# Unpair Cloud
Write-Host $("Started Cloud Unpairing") -ForegroundColor Green
$currentJob = Start-AzureSiteRecoveryProtectionProfileDissociationJob -ProtectionProfile $ProtectionProfile -PrimaryProtectionContainer $PrimaryContainer -RecoveryProtectionContainer $RecoveryContainer
$currentJob
WaitForJobCompletion -JobId $currentJob.Name -JobQueryWaitTimeInSeconds $JobQueryWaitTimeInSeconds
Write-Host $("****************Clouds Unpaired****************") -ForegroundColor Green

# Remove Protection Profile
Write-Host $("Started Protection Profile removal") -ForegroundColor Green
$currentJob = Remove-AzureSiteRecoveryProtectionProfile -Name $ProtectionProfileName
$currentJob
WaitForJobCompletion -JobId $currentJob.Name -JobQueryWaitTimeInSeconds $JobQueryWaitTimeInSeconds
Write-Host $("****************Removed Protection Profile:****************") -ForegroundColor Green