##### global environment initialization ######
	function Initialize-CSPVariables
	{
		
		if ($providerCred -eq $null) {$global:providerCred = Get-Credential -Message "Please provide your CSP Partner credentials:"}
		#Resellers tenant id, locate this in https://Partnercenter.microsoft.com, Account settings,  Organization Profile, Microsoft ID
		if ($providerAD -eq $null) {$global:providerAD = Connect-MsolService -Credential $providerCred}
		
		#Resellers tenant id, locate this in https://Partnercenter.microsoft.com, Account settings,  Organization Profile, Microsoft ID
		$global:ResellerTenantId = (Get-MsolPartnerInformation).ObjectId.Guid

		#As configured in AD for my Service Provider tenant (or through CSP panel)
		if ($ApplicationID -eq $null) {$global:ApplicationID = [system.uri]::EscapeDataString((Get-Credential -Message "Insert you CSP Partner Application ID:" -UserName "ApplicationId").GetNetworkCredential().Password)}

		#Application secret as defined in AD for my ApplicationID
		if ($ApplicationSecret -eq $null) {$global:ApplicationSecret = [system.uri]::EscapeDataString((Get-Credential -Message "Insert you CSP Partner API Key:" -UserName "API Key").GetNetworkCredential().Password)}

		#Reseller domain name
		$global:ResellerDomain = (Get-MsolDomain).name

		#Define common URI
		$global:URI_AAD="https://login.windows.net/$ResellerDomain/oauth2/token?api-version=1.0"                    
		$global:URI_GetCustomers="https://graph.windows.net/$ResellerDomain/contracts?api-version=1.5"
		$global:URI_CSPAPI="https://api.cp.microsoft.com/"
		$global:URI_GetSAToken=$URI_CSPAPI + "my-org/tokens"
		$global:URI_GetReseller=$URI_CSPAPI+"customers/get-by-identity?provider=AAD&type=tenant&tid=" + $ResellerTenantId
		$global:URI_CreateCustomer=$URI_CSPAPI + $ResellerTenantId + "/customers/create-reseller-customer"
		$global:URI_GetSubscriptions=$URI_CSPAPI+ $ResellerTenantId + "/subscriptions?recipient_customer_id="
	}
    
	################ Tokens #############################
	
	# Get the Azure Active Directory application token https://msdn.microsoft.com/en-us/library/partnercenter/dn974935.aspx
	# Cloud Solution Provider partners must generate their own authentication credentials— a client ID and a secret key—before they can work with the CREST APIs. They use these credentials to create an Azure Active Directory security token. 
	function Get-AADToken
	{
		$headers="" 
		$grantbody="grant_type=client_credentials&resource=https://graph.windows.net&client_id=$ApplicationID&client_secret=$ApplicationSecret"
		$AADTokenResponse=Invoke-Restmethod -Uri $URI_AAD -ContentType "application/x-www-form-urlencoded" -Body $grantbody -Method "POST" -verbose -Debug
		$global:AADToken = $AADTokenResponse.access_token
	}

	#GET SA token https://msdn.microsoft.com/en-us/library/partnercenter/mt146414.aspx
	#To use the CREST API, you must have an authorization token for the resellers account, which is generated using your Azure AD security token. This reseller token is called a Sales Agent Token, shortened to SA_Token.
	#After you create an SA_Token, you can use it to get your cid-for-reseller. See Get a reseller id.
	function Get-SAToken
	{
		Get-AADToken
		$headers=@{Authorization="Bearer $AADToken"
				}
		$SABody ="grant_type=client_credentials"
		$SATokenResponse = Invoke-RestMethod -Uri $URI_GetSAToken -ContentType "application/x-www-form-urlencoded" -Headers $headers -Body $SABody -method "POST" -Debug -Verbose
		$global:SAToken=$SATokenResponse.access_token
	}
	
	#### GET Customer Token #####
	function Get-CustomerToken(
		[String]$CustomerID
	)
	{
		Get-AADToken
		$CTokenUri = "https://api.cp.microsoft.com/$CustomerID/tokens"
		$headers=@{Authorization="Bearer $AADToken"}
		$SABody ="grant_type=client_credentials"
		$SATokenResponse = Invoke-RestMethod -Uri $CTokenUri -ContentType "application/x-www-form-urlencoded" -Headers $headers -Body $SABody -method "POST" -Debug -Verbose
		$global:CustomerToken=$SATokenResponse.access_token
	}
	
	
	########################################################
	###### RESELLER #######
	#Get a reseller id https://msdn.microsoft.com/en-us/library/partnercenter/mt427345.aspx
	#You can get the Customer resource that represents you, the CSP partner. This Customer resource contains an id which is your {cid-for-reseller} value for CREST API calls.
	#Before you can get a reseller ID, you must have an SA_Token. Get a reseller token.
	function Get-ResellerIdentity
	{
		Get-SAToken
		$TrackingGUID=[guid]::NewGuid()
		$CorrelationGUID=[guid]::NewGuid()
		$headers=@{Authorization="Bearer $SAToken"
				"Accept"="application/json"
				"api-version"="2015-03-31"
				"x-ms-correlation-id"=$CorrelationGUID
				"x-ms-tracking-id"=$TrackingGUID
				}

		$GetResellerResponse=Invoke-RestMethod -Uri $URI_GetReseller -ContentType "application/x-www-form-urlencoded" -Headers $headers -Method "GET" -Debug -Verbose
		$global:ResellerIdentity=$GetResellerResponse.id  #contains {cid-for-reseller}
		$ResellerIdentity
	}
	
    #get all my customer contracts
    
    
    function Get-Contracts
    {
        $headers=@{Authorization="Bearer $AADToken"
        }
        $SAGetCustomersResponse=Invoke-RestMethod -Uri $URI_GetCustomers -Headers $headers -ContentType "application/json" -Method "GET" -Debug -Verbose
        $SACustomers=$SAGetCustomersResponse.value
        $SACustomers
    }

	#### CUSTOMER METHODS#######
	# Create a customer https://msdn.microsoft.com/en-us/library/partnercenter/mt146403.aspx
	# Creates a new customer. As a CSP partner, you can place orders on behalf of this customer. It also creates the following:
	# An Azure AD tenant object that represents the customer. 
	# A Customer resource that represents the partner's security group in the customer's tenant. This is also the resource that is returned from the API. 
	# Appropriate relationships to create a reseller-customer relationship. 
	# A user name and password to login to the Microsoft Online Services Portal. 
	# Note that you should save the customer ID and Azure AD identity for your own records, in a way that will persist beyond the current session. Otherwise you will not have the information required by the CREST API for later account management.
	# If you do not have a customer's ID or identity, you can look up the ID in Partner Center by choosing the customer from the customers list, selecting Account, then saving their Microsoft ID.#

	function Create-Customer([string]$DomainNamePrefix,[string]$UserName,[string]$Password,[string]$Email,[string]$CompanyName,[string]$FirstName,[string]$LastName)
	{
		$customerdata= @{
			domain_prefix = $DomainNamePrefix;
			user_name = $UserName; 
			password = $Password;
			profile=@{
				email = $Email;
				company_name = $CompanyName;
				culture = "en-US";
				language = "en";
				type = "organization";
				default_address=@{
					first_name = $FirstName;
					last_name = $LastName;
					address_line1 = "Test Address";
					city = "Bellevue";
					region = "WA";
					postal_code = "98005";
					country = "US";
					phone_number = "4255551234"
				}
			
			}
		}

		$customerjson=$customerdata | ConvertTo-Json

		$TrackingGUID=[guid]::NewGuid()
		$CorrelationGUID=[guid]::NewGuid()
		$headers=@{Authorization="Bearer $SAToken"
				"Accept"="application/json"
				"api-version"="2015-03-31"
				"x-ms-correlation-id"=$CorrelationGUID
				"x-ms-tracking-id"=$TrackingGUID
				}
		

		$customer=Invoke-RestMethod -Uri $URI_CreateCustomer -Method "POST" -Headers $headers -Body $customerjson -ContentType "application/json"
	 }    $customer
	 
	 #### GET Customer By ID
	 function Get-CustomerByID([String]$CustomerID)
	{
		Get-SAToken
		$TrackingGUID=[guid]::NewGuid()
		$CorrelationGUID=[guid]::NewGuid()
		$headers=@{Authorization="Bearer $SAToken"
				"Accept"="application/json"
				"api-version"="2015-03-31"
				"x-ms-correlation-id"=$CorrelationGUID
				"x-ms-tracking-id"=$TrackingGUID
				}
		#$APIUri = "https://api.cp.microsoft.com/customers/$CustomerID"
		$APIUri = "https://api.cp.microsoft.com/customers/get-by-identity?provider=AAD&type=external_group&tid=$CustomerID&etid=$ResellerTenantId"
		$GetCusromerResponse=Invoke-RestMethod -Uri $APIUri -ContentType "application/x-www-form-urlencoded" -Headers $headers -Method "GET" -Debug -Verbose
		$GetCusromerResponse
	}
	
	#### GET Customer Profile
	function Get-CustomerProfile([String]$CustomerID)
	{
		Get-CustomerToken -CustomerID $CustomerID
		$headers=@{Authorization="Bearer $CustomerToken"
				"Accept"="application/json"
				"api-version"="2015-03-31"
				}
		$APIUri = "https://api.cp.microsoft.com/$CustomerID/profiles"
		$GetCusromerResponse=Invoke-RestMethod -Uri $APIUri -ContentType "application/x-www-form-urlencoded" -Headers $headers -Method "GET" -Debug -Verbose
		$GetCusromerResponse
	}
	
	################## SUBSCRIPTION METHODS 
	
	 ##### Get All Subscriptions for a customer
	 function Get-AllSubscriptions([String]$CustomerID)
	{
		$TrackingGUID=[guid]::NewGuid()
		$CorrelationGUID=[guid]::NewGuid()
		$headers=@{Authorization="Bearer $SAToken"
				"Accept"="application/json"
				"api-version"="2015-03-31"
				"x-ms-correlation-id"=$CorrelationGUID
				"x-ms-tracking-id"=$TrackingGUID
				}
		$APIUri = "https://api.cp.microsoft.com/$ResellerTenantId/subscriptions?recipient_customer_id=$CustomerID"
		$GetSubscriptionResponse = Invoke-RestMethod -Uri $APIUri -ContentType "application/x-www-form-urlencoded" -Headers $headers -Method "GET" -Debug -Verbose
		$GetSubscriptionResponse
	}
	
	##### Get  Subscription by ID
	function Get-Subscription([String]$SubscriptionID)
	{
		$TrackingGUID=[guid]::NewGuid()
		$CorrelationGUID=[guid]::NewGuid()
		$headers=@{Authorization="Bearer $SAToken"
				"Accept"="application/json"
				"api-version"="2015-03-31"
				"x-ms-correlation-id"=$CorrelationGUID
				"x-ms-tracking-id"=$TrackingGUID
				}
		$APIUri = "https://api.cp.microsoft.com/$ResellerTenantId/subscriptions/$SubscriptionID"
		$GetSubscriptionResponse = Invoke-RestMethod -Uri $APIUri -ContentType "application/x-www-form-urlencoded" -Headers $headers -Method "GET" -Debug -Verbose
		$GetSubscriptionResponse
	}
	
	##### Remove Subscription suspension
	function Remove-SubscriptionSuspension(
		[String]$SubscriptionID
	)
	{
		Get-SAToken
		$TrackingGUID=[guid]::NewGuid()
		$CorrelationGUID=[guid]::NewGuid()
		$headers=@{Authorization="Bearer $SAToken"
				"Accept"="application/json"
				"api-version"="2015-03-31"
				"x-ms-correlation-id"=$CorrelationGUID
				"x-ms-tracking-id"=$TrackingGUID
				}
		$RData = @{
			suspension_reason = "CustomerCancellation"
		}
		$RBodyJson=$RData | ConvertTo-Json
		 
		$SBody = "suspension_reason = CustomerCancellation"
		$APIUri = "https://api.cp.microsoft.com/$ResellerTenantId/subscriptions/$SubscriptionID/remove-suspension"
		$GetSubscriptionResponse = Invoke-RestMethod -Uri $APIUri -ContentType "application/json" -Headers $headers -Method "POST" -Body $RBodyJson -Debug -Verbose
		$GetSubscriptionResponse
	}
	
	##### Suspend Subscription
	function Suspend-Subscription(
		[String]$SubscriptionID,
		[String]$SuspensionComment
		)
	{
		Get-SAToken
		$TrackingGUID=[guid]::NewGuid()
		$CorrelationGUID=[guid]::NewGuid()
		$headers=@{Authorization="Bearer $SAToken"
				"Accept"="application/json"
				"api-version"="2015-03-31"
				"x-ms-correlation-id"=$CorrelationGUID
				"x-ms-tracking-id"=$TrackingGUID
				}
		$RData = @{
			suspension_reason = "CustomerCancellation"
			suspension_reason_comment = $SuspensionComment
		}
		$RBodyJson=$RData | ConvertTo-Json
		
		$SBody = "suspension_reason = CustomerCancellation"
		$APIUri = "https://api.cp.microsoft.com/$ResellerTenantId/subscriptions/$SubscriptionID/add-suspension"
		$GetSubscriptionResponse = Invoke-RestMethod -Uri $APIUri -ContentType "application/json" -Headers $headers -Method "POST" -Body $RBodyJson -Debug -Verbose
		$GetSubscriptionResponse
	}
	
	##### Get Subscription Sources
	function Get-SubscriptionSource([String]$SubscriptionID)
	{
		$TrackingGUID=[guid]::NewGuid()
		$CorrelationGUID=[guid]::NewGuid()
		$headers=@{Authorization="Bearer $SAToken"
				"Accept"="application/json"
				"api-version"="2015-03-31"
				"x-ms-correlation-id"=$CorrelationGUID
				"x-ms-tracking-id"=$TrackingGUID
				}
		$APIUri = "https://api.cp.microsoft.com/$ResellerTenantId/subscriptions/$SubscriptionID/sources "
		$GetSubscriptionResponse = Invoke-RestMethod -Uri $APIUri -ContentType "application/x-www-form-urlencoded" -Headers $headers -Method "GET" -Debug -Verbose
		$GetSubscriptionResponse
	}
	
	###### Get All orders#######
	function Get-AllOrders([String]$CustomerID)
	{
		Get-SAToken
		$headers=@{Authorization="Bearer $SAToken"
				"Accept"="application/json"
				"api-version"="2015-03-31"
				}
		$APIUri = "https://api.cp.microsoft.com/$ResellerTenantId/orders?recipient_customer_id=$CustomerID"
		$GetCusromerResponse=Invoke-RestMethod -Uri $APIUri -ContentType "application/x-www-form-urlencoded" -Headers $headers -Method "GET" -Debug -Verbose
		$GetCusromerResponse
	}
	
	#### GET Customer Entitlements
	function Get-Entitlements([String]$CustomerID)
	{
		$res = Get-CustomerToken -CustomerID $CustomerID
		$headers=@{Authorization="Bearer $CustomerToken"
				"Accept"="application/json"
				"api-version"="2015-03-31"
				}
		$APIUri = "https://api.cp.microsoft.com/$CustomerID/Entitlements"
		$GetCusromerResponse=Invoke-RestMethod -Uri $APIUri -ContentType "application/x-www-form-urlencoded" -Headers $headers -Method "GET" -Debug -Verbose
		$GetCusromerResponse
	}
	
	
	####################### USAGE METHODS
	
	##### Get All Usage Records for Customer Entitlements from created date to supplied EndDate- CustomerID, EndDate 
	function Get-AllUsageFromCustomerEntitlements(
		[String]$CustomerID,
		[String]$EndDate
	)
	{
		#$CustomerID = "9bf6ce16-f2d2-43ee-ac11-eb2c3846527c"
		Get-CustomerToken -CustomerID $CustomerID
		$headers=@{Authorization="Bearer $CustomerToken"
				"Accept"="application/json"
				"api-version"="2015-03-31"
				}
		$APIUri = "https://api.cp.microsoft.com/$CustomerID/Entitlements"
		$GetCusromerResponse=Invoke-RestMethod -Uri $APIUri -ContentType "application/x-www-form-urlencoded" -Headers $headers -Method "GET" -Debug -Verbose
		#$GetCusromerResponse
		
		Get-SAToken
		$headers=@{Authorization="Bearer $SAToken"
					"Accept"="application/json"
					"api-version"="2015-03-31"
					}
		foreach ($e in $GetCusromerResponse.Items)
		{
			$sdate = $e.start_date
			$eID = $e.Id
			#$edate= "2015-08-28T00:00:00Z"
			$edate = $EndDate
			
			$APIUri = "https://api.cp.microsoft.com/$ResellerTenantId/usage-records?entitlement_id=$eID&reported_start_time=$sdate&reported_end_time=$edate&show_details=True&granularity=daily"
			$GetCusromerResponse1=Invoke-RestMethod -Uri $APIUri -ContentType "application/x-www-form-urlencoded" -Headers $headers -Method "GET" -Debug -Verbose
			$GetCusromerResponse1.items
		}
	}

    